@@ -279,7 +279,7 @@ class CSVLoader:
             if len(self.levels_shifts_inter) == 0:
                 self.shifts_inter = None
             else:
-                self.shifts_inter = Shifts(self.ranges_shifts_inter, self.levels_shifts_inter, self.n_samples)
+                self.shifts_inter = Shifts(self.ranges_shifts_inter, self.levels_shifts_inter, self.n_samples_e)
                 self.levels_shifts_inter = self.shifts_inter.levels
                 self.levels_shifts_inter_sort = self.shifts_inter.levels_sort
                 self.ranges_shifts_inter = self.shifts_inter.ranges
@@ -287,7 +287,7 @@ class CSVLoader:
             if len(self.levels_shifts_intra) == 0:
                 self.shifts_intra = None
             else:
-                self.shifts_intra = Shifts(self.ranges_shifts_intra, self.levels_shifts_intra, self.batch_size)
+                self.shifts_intra = Shifts(self.ranges_shifts_intra, self.levels_shifts_intra, self.n_samples_e)
                 self.levels_shifts_intra = self.shifts_intra.levels
                 self.levels_shifts_intra_sort = self.shifts_intra.levels_sort
                 self.ranges_shifts_intra = self.shifts_intra.ranges
@@ -451,30 +451,9 @@ class CSVLoader:
         print()
 
     def __iter__(self):
-        return self
-
-    def __next__(self):
-
-        """
-        short description
-
-        long description
-
-        Returns
-        -------
-        out : generator
-            A generator of lists of elements requested in order_outputs. The elements have the same order as in
-            order_outputs. It generates a list per batch of input samples.
-
-        Raises
-        ------
-        ValueError
-            something
-
-        """
 
         if self.shifts_inter is not None:
-            self.shifts_inter.next()
+            self.shifts_inter.refresh()
             # TODO: recopy only the variable conditions with shifts
             self.combinations_directories_inter = np.copy(self.combinations_directories_inter_no_shift)
             self.combinations_directories_inter[:, self.shifts_inter.levels] += self.shifts_inter.values
@@ -486,7 +465,7 @@ class CSVLoader:
                 self.labels = None
 
         if self.shifts_intra is not None:
-            self.shifts_intra.next()
+            self.iter_shifts_intra = iter(self.shifts_intra)
 
         if self.shuffle:
             indexes_samples_e, self.bin_indexes_samples = np.split(
@@ -504,23 +483,43 @@ class CSVLoader:
             self.size_bin_indexes_samples = len(self.bin_indexes_samples)
             self.batches_indexes = np.split(indexes_samples_e, self.n_batches, axis=0)
 
-        # for b in range(1):
-        for b in range(self.n_batches):
+        self.b = -1
+        self.s = -1
+
+        return self
+
+    def __next__(self):
+        """
+
+        Returns
+        -------
+
+        """
+
+        self.b += 1
+
+        # if self.b < 1:
+        if self.b < self.n_batches:
+
             if self.return_labels_eb:
-                labels_eb = self.labels[self.batches_indexes[b]]
+                labels_eb = self.labels[self.batches_indexes[self.b]]
+
+            combinations_inter_eb = self.combinations_directories_inter[self.batches_indexes[self.b], :]
 
-            combinations_inter_eb = self.combinations_directories_inter[self.batches_indexes[b], :]
             # batch_size_eb = batches_indexes_e.shape[0]
             for i in range(self.batch_size):
 
+                self.s += 1
+
                 self.indexes_batch[self.batch_axis_inputs] = i
                 self.indexes_directories_ebij[self.batch_axis_directories] = i
 
                 if self.shifts_intra is not None:
+                    shifts_intra_s = np.expand_dims(next(self.iter_shifts_intra), axis=0)
+
                     # TODO: recopy only the variable conditions with shifts
                     self.combinations_directories_intra = np.copy(self.combinations_directories_intra_no_shift)
-                    self.combinations_directories_intra[:, self.shifts_intra.levels] += (
-                        self.shifts_intra.values[slice(i, i + 1, 1), :])
+                    self.combinations_directories_intra[:, self.shifts_intra.levels] += shifts_intra_s
 
                 combination_inter_ebi = combinations_inter_eb[i, :]
 
@@ -572,8 +571,8 @@ class CSVLoader:
 
                         array_np_ebij = cp_txt.csv_file_to_array(
                             absolute_directory_ebij, rows=self.rows, columns=self.columns, dtype='f')
-
                         tensor_ebij = torch.tensor(array_np_ebij, dtype=torch.float32, device=self.device)
+
                         # todo move dimensions of tensor_ebi
                         #  if csv_axes_inputs[0] > csv_axes_inputs[1]
 
@@ -598,3 +597,5 @@ class CSVLoader:
                     self.outputs[o] = self.absolute_directories_eb
 
             return self.outputs
+        else:
+            raise StopIteration
